#!/usr/bin/env python

'''
Server mode using Gevent, using tunnelling feature to give a remote shell.
'''

from __future__ import absolute_import

import os
import sys
import fcntl

from sjrpc.server import RpcServer, SSLRpcServer
from sjrpc.utils import RpcHandler, pass_connection
from sjrpc.core.protocols import TunnelProtocol

import logging
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
fmt = logging.Formatter('%(levelname)s %(name)s %(message)s')
handler = logging.StreamHandler()
handler.setFormatter(fmt)
logger.addHandler(handler)


class _FakeDevZeroSocket(object):

    def __init__(self):
        self._devzero = open('/dev/zero', 'rb')

    def recv(self, size):
        return '\0' * size#self._devzero.read(size)

    def send(self, data):
        pass # Cannot write on /dev/zero

    def fileno(self):
        return self._devzero.fileno()

    def setblocking(self, blocking):
        if not blocking:
            # Disable blocking mode on devzero:
            current_flags = fcntl.fcntl(self.fileno(), fcntl.F_GETFL)
            fcntl.fcntl(self.fileno(), fcntl.F_SETFL, current_flags | os.O_NONBLOCK)
            pass
        else:
            raise NotImplementedError('Not implemented')


class PerfHandler(RpcHandler):

    @pass_connection
    def launch_perf(self, connection):
        devzero = _FakeDevZeroSocket()
        connection.register_protocol(2, TunnelProtocol, endpoint=devzero)
        return 2


# Get arguments from the command line:
if len(sys.argv) < 3:
    print 'Usage: %s <listening address> <listening port>' % sys.argv[0]
    sys.exit(2)
address = sys.argv[1]
port = int(sys.argv[2])

# Create the server instance:
import pyev
loop = pyev.default_loop(flags=~ pyev.EVBACKEND_EPOLL)
rpcserver = SSLRpcServer.from_addr(address, port, loop=loop, certfile='/var/lib/cc-server/ccserver.crt', keyfile='/var/lib/cc-server/ccserver.key', conn_kw=dict(handler=PerfHandler()))
#rpcserver = RpcServer.from_addr(address, port, loop=loop, conn_kw=dict(handler=PerfHandler()))
rpcserver.run()

