# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.


""" Contains sjRpc exceptions.
"""


class RpcError(Exception):

    """ Exception raised by caller when an error occurs while execution of
        the remote procedure call.
    """

    def __init__(self, exception, message):
        self.exception = exception
        self.message = message

    def __str__(self):
        return '%s' % self.message


class RpcConnectionError(Exception):

    """ Base class for RpcConnection errors.
    """


class SocketError(RpcConnectionError):

    """ Exception used internally to raise a socket fault.
    """


class NoFreeLabelError(RpcConnectionError):

    """ Exception raised when no more free labels are available for protocol
        allocation.
    """


class FallbackModeEnabledError(RpcConnectionError):

    """ Exception raised when a feature which is not compatible with fallback
        mode is used.
    """


class AlreadyAnsweredError(Exception):

    """ Exception raised when a deferred response has already been answered.
    """