# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.


from __future__ import absolute_import

from sjrpc.core.exceptions import RpcError

class ConnectionProxy(object):

    """ Create a new call proxy for the connection passed in arguments.

    Usage example:

    >>> proxy = ConnectionProxy(conn)
    >>> ret = proxy.existing_function()

    If exception is raised by the function, the proxy tries to get the
    exception class in *__builtins__* to re-raise it. If exception class
    is not found in *__builtins__*, the exception raised is the
    original :exc:`RpcError`:

    >>> proxy.unknown_function()
    [Traceback]
    NameError: remote name 'unknown_function' is not defined

    >>> proxy['myfunc']() # You can also use this syntax
    ...
    """

    def __init__(self, connection):
        self.connection = connection

    def __getattr__(self, name):
        def func(*args, **kwargs):
            try:
                returned = self.connection.call(name, *args, **kwargs)
            except RpcError as err:
                expt = getattr(__builtins__, err.exception)
                if expt is not None:
                    raise expt(err.message)
                else:
                    raise err
            else:
                return returned

        return func

    def __getitem__(self, name):
        return self.__getattr__(name)
